////////////////////////////////////////////////////////////////////////////////
// Part of Internet EDiting component                                         //
// AUTHOR       : Sergey Kanenko                                              //
// WEBSITE      : http://iedcomp.nm.ru                                        //
////////////////////////////////////////////////////////////////////////////////
//   This code may be used and modified by anyone so long as  this header and //
// copyright  information remains intact.                                     //
//   The code is provided "as-is" and without warranty of any kind,           //
// expressed, implied or otherwise, including and without limitation, any     //
// warranty of merchantability or fitness for a  particular purpose.         //
//   In no event shall the author be liable for any special, incidental,      //
// indirect or consequential damages whatsoever (including, without           //
// limitation, damages for loss of profits, business interruption, loss       //
// of information, or any other loss), whether or not advised of the          //
// possibility of damage, and on any theory of liability, arising out of      //
// or in connection with the use or inability to use this software.         //
////////////////////////////////////////////////////////////////////////////////
unit UnDAVmain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, 
  IEDResManager, IEDXML, ExtCtrls, Buttons;

type
  TfrmMain = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    tv: TTreeView;
    Memo2: TMemo;
    Panel1: TPanel;
    Memo3: TMemo;
    btnGo: TButton;
    Memo1: TMemo;
    TabSheet3: TTabSheet;
    tr: TListView;
    Label1: TLabel;
    Label2: TLabel;
    edUser: TEdit;
    edPwd: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    Memo4: TMemo;
    Label5: TLabel;
    Label6: TLabel;
    edPort: TEdit;
    Label7: TLabel;
    Bevel1: TBevel;
    edPath: TEdit;
    Label8: TLabel;
    btnUp: TSpeedButton;
    btnMKcol: TButton;
    btnDelete: TButton;
    btnUpload: TButton;
    OpenDialog1: TOpenDialog;
    btnDownload: TButton;
    edHost: TComboBox;
    cbAutoRefresh: TCheckBox;
    Button1: TButton;
    btnLOCK: TButton;
    btnUNLOCK: TButton;
    SaveDialog1: TSaveDialog;
    StatusBar1: TStatusBar;
    procedure btnGoClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure trCompare(Sender: TObject; Item1, Item2: TListItem;
      Data: Integer; var Compare: Integer);
    procedure trDblClick(Sender: TObject);
    procedure btnUpClick(Sender: TObject);
    procedure btnMKcolClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnUploadClick(Sender: TObject);
    procedure btnDownloadClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure btnLOCKClick(Sender: TObject);
    procedure btnUNLOCKClick(Sender: TObject);
  private
    procedure ShowResponseTree(Parser: TIEDXMLReader);
    procedure ShowResponseReport;
    procedure UpdateResponses(DoRefresh:boolean);
    function SelectWebDavDocument: boolean;
    { Private declarations }
  public
    { Public declarations }
    WebDav : TIEDWebDav;
  end;

var
  frmMain: TfrmMain;

implementation


{$R *.dfm}

procedure TfrmMain.btnGoClick(Sender: TObject);
begin
 WebDav.Host := edHost.Text;
 WebDav.User := edUser.Text;
 WebDav.Password := edPwd.Text;
 WebDav.Port := StrToInt(edPort.Text);
 WebDav.CurrentDir := edPath.Text;

 WebDav.Connect;
 WebDav.LoadResourceList;
 WebDav.Content.Position :=0;
 UpdateResponses(false);
 ShowResponseTree(WebDav.XMLreader);
 ShowResponseReport;
end;

procedure TfrmMain.UpdateResponses;
begin
 Memo4.Lines.Assign(WebDav.CHeader);
 Memo3.Lines.LoadFromStream(WebDav.Content);
 Memo1.Lines.Assign(WebDav.RHeader);
 WebDav.Response.Position := 0;
 Memo2.Lines.LoadFromStream(WebDav.Response);
 if DoRefresh and cbAutoRefresh.Checked then
    btnGoClick(nil);
end;

function TfrmMain.SelectWebDavDocument:boolean;
var  i : integer;
begin
 result:= false;
 if tr.ItemIndex>-1 then
  begin
   i:= TResource(tr.Items[tr.ItemIndex].Data).Index;
   if i<0 then exit;
   WebDav.SelectedIndex := i;
   result:= true;
  end;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
 WebDav := TIEDWebDav.Create;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
 WebDav.Free;
end;

procedure TfrmMain.ShowResponseTree(Parser : TIEDXMLReader);

 procedure AddViewChild(t:TTreeNode; p: TIEDXMLNode);
 var i : integer;
 begin
  if not Assigned(p) then exit;
  t:=tv.Items.AddChild(t,p.Text);
  t.Data := p;
  for i:=0 to p.Nodes.Count-1 do
   AddViewChild(t,p.Nodes[i]);
  t.Expanded := true;
 end;

begin
 try
  tv.Items.BeginUpdate;
  tv.Items.Clear;
  AddViewChild(tv.Items.GetFirstNode,Parser.Root);
 finally
  tv.Items.EndUpdate;
 end;
end;

procedure TfrmMain.ShowResponseReport;
var i : integer;
begin
 Self.Cursor := crHourGlass;
try
 tr.Clear;
 for i:=0 to WebDav.ResourcesCount-1 do
 With tr.Items.Add do
   With WebDav.Resources[i] do
    begin
     Data    := WebDav.Resources[i];
     Caption := DisplayName;
     if IsCollection then
      begin
       SubItems.Add('Dir');
       SubItems.Add('');
      end
     else
      begin
       SubItems.Add('');
       SubItems.Add(IntToStr(rspContentLength));
      end;
     SubItems.Add(LockEntry.lockOwner);
     SubItems.Add(DateTimeToStr(rspCreationDate));
     SubItems.Add(DateTimeToStr(rspLastModified));
     SubItems.Add(ResourceName);
     SubItems.Add(StatusStr);
     SubItems.Add(rspHREF);
  end;
 finally
   Self.Cursor := crDefault;
 end;
end;

procedure TfrmMain.trCompare(Sender: TObject; Item1, Item2: TListItem;
  Data: Integer; var Compare: Integer);
begin
 if ord(TResource(Item1.Data).IsCollection)>ord(TResource(Item2.Data).IsCollection)
  then Compare :=-1
 else
 if ord(TResource(Item1.Data).IsCollection)<ord(TResource(Item2.Data).IsCollection)
  then Compare := 1
 else
   Compare := CompareText(Item1.Caption,Item2.Caption);
end;

procedure TfrmMain.trDblClick(Sender: TObject);
var ResData : TResource;
begin
 if SelectWebDavDocument then
  With WebDav.SelectedResource do
   if (IsCollection and IsOK) then
    begin
     if DisplayName='.' then     // ;)
      edPath.Text:= UpDirectory(edPath.Text)
     else
      edPath.Text:= ResourceName;
     btnGoClick(nil);
    end;
end;

procedure TfrmMain.btnUpClick(Sender: TObject);
begin
  edPath.Text:= UpDirectory(edPath.Text);
  btnGoClick(nil);
end;

procedure TfrmMain.btnMKcolClick(Sender: TObject);
var DirName : string;
begin
 if InputQuery('Create new directory','Directory name',DirName) then
 begin
  WebDav.MkDir(DirName);
  UpdateResponses(True);
 end;
end;

procedure TfrmMain.btnDeleteClick(Sender: TObject);
begin
 if SelectWebDavDocument then
  begin
   WebDav.Delete;
   UpdateResponses(True);
  end;
end;

procedure TfrmMain.btnUploadClick(Sender: TObject);
begin
 if OpenDialog1.Execute then
   WebDav.Upload(OpenDialog1.FileName);
 UpdateResponses(True);
end;

procedure TfrmMain.btnDownloadClick(Sender: TObject);
begin
 if SelectWebDavDocument then
  begin
   SaveDialog1.FileName := WebDav.SelectedResource.DisplayName;
   if SaveDialog1.Execute then
    WebDav.Download(SaveDialog1.FileName);
   UpdateResponses(True);
  end;
end;

procedure TfrmMain.Button1Click(Sender: TObject);
begin
 WebDav.Disconnect;
end;

procedure TfrmMain.btnLOCKClick(Sender: TObject);
begin
 if SelectWebDavDocument then
  begin
   WebDav.LOCK('My Lock');
   UpdateResponses(True);
  end;
end;

procedure TfrmMain.btnUNLOCKClick(Sender: TObject);
begin
 if SelectWebDavDocument then
  begin
   WebDav.UNLOCK;
   UpdateResponses(True);
  end;
end;

end.
