////////////////////////////////////////////////////////////////////////////////
// Part of Internet EDiting component                                         //
// AUTHOR       : Sergey Kanenko                                              //
// WEBSITE      : http://iedcomp.nm.ru                                        //
////////////////////////////////////////////////////////////////////////////////
//   This code may be used and modified by anyone so long as  this header and //
// copyright  information remains intact.                                     //
//   The code is provided "as-is" and without warranty of any kind,           //
// expressed, implied or otherwise, including and without limitation, any     //
// warranty of merchantability or fitness for a  particular purpose.         //
//   In no event shall the author be liable for any special, incidental,      //
// indirect or consequential damages whatsoever (including, without           //
// limitation, damages for loss of profits, business interruption, loss       //
// of information, or any other loss), whether or not advised of the          //
// possibility of damage, and on any theory of liability, arising out of      //
// or in connection with the use or inability to use this software.         //
////////////////////////////////////////////////////////////////////////////////
unit UnXMLMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, IEDXML;

type
  TfrmXMLmain = class(TForm)
    Panel1: TPanel;
    Memo2: TMemo;
    tv: TTreeView;
    Splitter1: TSplitter;
    Panel2: TPanel;
    Button2: TButton;
    Button1: TButton;
    OpenDialog1: TOpenDialog;
    Button3: TButton;
    Edit1: TEdit;
    cbExp: TCheckBox;
    Button4: TButton;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
  private
    procedure ShowTreeFromStream(MS: TStream);
    { Private declarations }
  public
    { Public declarations }
    Parser : TIEDXMLReader;
    schNode: TIEDXMLNode;
  end;

var
  frmXMLmain: TfrmXMLmain;

implementation

{$R *.dfm}

procedure TfrmXMLmain.FormCreate(Sender: TObject);
begin
 Parser := TIEDXMLReader.Create;
end;

procedure TfrmXMLmain.FormDestroy(Sender: TObject);
begin
 Parser.Free;
end;

procedure TfrmXMLmain.Button1Click(Sender: TObject);
begin
 if OpenDialog1.Execute then
  Memo2.Lines.LoadFromFile(OpenDialog1.FileName);
end;

procedure TfrmXMLmain.ShowTreeFromStream(MS : TStream);

 procedure AddViewChild(t:TTreeNode; p: TIEDXMLNode);
 var i : integer;
 begin
  if not Assigned(p) then exit;
  t:=tv.Items.AddChild(t,p.Text);
  t.Data := p;
  for i:=0 to p.Nodes.Count-1 do
   AddViewChild(t,p.Nodes[i]);
  t.Expanded := cbExp.Checked;
 end;

begin
 try
  MS.Seek(0,soFromBeginning);
  tv.Items.BeginUpdate;
  tv.Items.Clear;
  parser.Root.Clear;
  Parser.Parse(MS);
  AddViewChild(tv.Items.GetFirstNode,parser.Root);
 finally
  tv.Items.EndUpdate;
 end;
end;

procedure TfrmXMLmain.Button3Click(Sender: TObject);
begin
 With tv do
 if (Selected<>nil)and(Selected.Data<>nil) then
   schNode:= TIEDXMLNode(Selected.Data).NodeOf(Edit1.Text,[schPartial,schUpper],100)
 else
   begin
    ShowMessage('Select node first');
    exit;
   end;

 if schNode<>nil then
  ShowMessage(schNode.Text)
 else
  ShowMessage('Not found');
end;

procedure TfrmXMLmain.Button2Click(Sender: TObject);
var MS     : TMemoryStream;
begin
  MS := TMemoryStream.Create;
  Memo2.Lines.SavetoStream(MS);
  ShowTreeFromStream(MS);
  MS.Free;
end;

procedure TfrmXMLmain.Button4Click(Sender: TObject);
var FS     : TFileStream;
begin
 if OpenDialog1.Execute then
 begin
   FS := TFileStream.Create(OpenDialog1.FileName,fmOpenRead);
   ShowTreeFromStream(FS);
   FS.Free;
 end;
end;

end.
