// HREFLinker Component  1.0                                                  //
////////////////////////////////////////////////////////////////////////////////
// Part of Internet EDiting component                                         //
// AUTHOR       : Sergey Kanenko                                              //
// WEBSITE      : http://iedcomp.nm.ru                                        //
////////////////////////////////////////////////////////////////////////////////
//   This code may be used and modified by anyone so long as  this header and //
// copyright  information remains intact.                                     //
//   The code is provided "as-is" and without warranty of any kind,           //
// expressed, implied or otherwise, including and without limitation, any     //
// warranty of merchantability or fitness for a  particular purpose.         //
//   In no event shall the author be liable for any special, incidental,      //
// indirect or consequential damages whatsoever (including, without           //
// limitation, damages for loss of profits, business interruption, loss       //
// of information, or any other loss), whether or not advised of the          //
// possibility of damage, and on any theory of liability, arising out of      //
// or in connection with the use or inability to use this software.         //
////////////////////////////////////////////////////////////////////////////////
unit HREFLinker;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  HREFParser, IdTCPConnection, IdFTP, IdHTTP;

type
  TOnDocumentParsed = procedure(Sender: TObject; Doc : THREFDocument;
                                LinksList : TStrings ) of object;
  TOnNewDocument = procedure( Sender: TObject; Link : THREF) of object;


  THREFLinker = class(TComponent)
  private
    FIdFTP: TIdFTP;
    FIdHTTP: TIdHTTP;
    FOnDocumentParsed: TOnDocumentParsed;
    FExtForShow : TExtentionSet;
    FOnFoundHREF: TOnFoundHREF;
    FOnFoundScript: TOnFoundScript;
    FExtForParse: TExtentionSet;
    FMaxParseLevel: integer;
    FOnNewDocument: TOnNewDocument;
    FLocalParse: boolean;
    FGlobalUniqueList : TStringList;
    FDoTerminate: boolean;
    FAddOnlyNew: boolean;
    function CheckUnique(AURL: string): boolean;
  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;
     function  LoadDataByLocal(AHREF : THREF; ALines:TStrings):boolean;
     function  LoadDataByFTP(AHREF : THREF; ALines:TStrings):boolean;
     function  LoadDataByHTTP(AHREF : THREF; ALines:TStrings):boolean;
     function  LoadDataByProtocol( AHREF : THREF; ALines:TStrings): boolean;
  public
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;
     procedure  ParseFileByHREF(AHREF: THREF);
     procedure  ParseFileByURL(AURL : string);
     property   DoTerminate: boolean write FDoTerminate;
  published
     property FTPLink  : TIdFTP read FIdFTP write FIdFTP;
     property HTTPLink : TIdHTTP read FIdHTTP write FIdHTTP;
     property ShowLinks   : TExtentionSet read FExtForShow write FExtForShow;
     property ParseDocument  : TExtentionSet read FExtForParse write FExtForParse;
     property MaxParseLevel : integer read FMaxParseLevel write FMaxParseLevel;
     property LocalParse : boolean read FLocalParse write FLocalParse default true;
     property AddOnlyNew : boolean read FAddOnlyNew write FAddOnlyNew default false;

     property OnNewDocument : TOnNewDocument Read FOnNewDocument Write FOnNewDocument;
     property OnDocumentParsed : TOnDocumentParsed Read FOnDocumentParsed Write FOnDocumentParsed;
     property OnFoundHyperlink : TOnFoundHREF Read FOnFoundHREF Write FOnFoundHREF;
     property OnFoundScript    : TOnFoundScript Read FOnFoundScript Write FOnFoundScript;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('IED', [THREFLinker]);
end;

{ THREFLinker }
constructor THREFLinker.Create;
begin
  inherited;
  FGlobalUniqueList := TStringList.Create;
  FGlobalUniqueList.Sorted:= true;
  FExtForShow   := [feHTML,feScript,feCSS,feIMG,feJava,feASP,fePHP,fePERL];
  FExtForParse  := [fePHP,feHTML,feScript];
  FLocalParse   := true;
  FMaxParseLevel:= 2;
end;

destructor THREFLinker.Destroy;
begin
 if Assigned(FGlobalUniqueList)  then FGlobalUniqueList.Free;
 inherited;
end;

procedure THREFLinker.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then
  begin
    if AComponent = FIdFTP then
      FIdFTP := nil
    else
    if AComponent = FIdHTTP then
      FIdHTTP := nil;
  end;
end;

function THREFLinker.CheckUnique(AURL: string):boolean;
var pp : integer;
begin
 result:=false;
  if FGlobalUniqueList.Find(AURL, pp) then
   exit
  else
   FGlobalUniqueList.AddObject(AURL, nil);
 result:=true;
end;

procedure THREFLinker.ParseFileByURL(AURL: string);
var HREFSelf : THREF;
begin
 FDoTerminate := false;
 FGlobalUniqueList.Clear;
 HREFSelf := THREF.Create(AURL);
 HREFSelf.Level :=0;
 try
  ParseFileByHREF(HREFSelf);
 finally
  if Assigned(HREFSelf) then HREFSelf.Free;
 end;
end;

procedure THREFLinker.ParseFileByHREF;
var
 i        : integer;
 HREFdoc  : THREFDocument;
 TL,
 TS       : TStringList;
begin
 Application.ProcessMessages;
 HREFdoc := THREFDocument.Create;
 TS      := TStringList.Create;
 TL      := TStringList.Create;
 HREFdoc.OnFoundHyperlink := FOnFoundHREF;
 HREFdoc.OnFoundScript := FOnFoundScript;
 try
 With HREFdoc do
 begin
   HREFSelf.Assign(AHREF);
   //Self Unique check
   CheckUnique(HREFSelf.UniqueDocURL);
   //Self add to diagram
   if Assigned(FOnNewDocument) then
    FOnNewDocument(self, HREFSelf);
   //Self Load
   if LoadDataByProtocol(HREFSelf, HTML) then
    begin
     Parse;
     for i:=0 to HREFList.Count-1 do
      HREFList[i].AssignBaseURL(HREFSelf);
    end;
   //Check Level
   if (AHREF.Level >= FMaxParseLevel)or(FDoTerminate) then
    exit;
   // Add links
   if (HREFSelf.State = lsGood)and
      (HREFSelf.GetFileType in FExtForParse) then
    begin
      //remove duplicate
      GetUniqueDocs(TS,FExtForShow);
      TL.Assign(TS);
      for i:=TS.Count-1 downto 0 do
       if not CheckUnique(TS[i]) then
        TS.Delete(i);
      //Go to next link
      for i:=0 to TS.Count-1 do
       with THREF(TS.Objects[i]) do
        if (GetFileType in FExtForShow) then
         begin
          Level:= HREFSelf.Level+1;
          ParseFileByHREF(TS.Objects[i] as THREF);
          if Assigned(FOnNewDocument) then
           FOnNewDocument(self, TS.Objects[i] as THREF);
         end;
    end;
 end;

 finally
   if Assigned(FOnDocumentParsed) then
    begin
      if FAddOnlyNew then
       FOnDocumentParsed(self, HREFdoc, TS)
      else
       FOnDocumentParsed(self, HREFdoc, TL);
    end;

  if Assigned(HREFdoc) then
   begin
    AHREF.Assign(HREFdoc.HREFSelf);
    HREFdoc.Free;
   end;
  TS.Free;
  TL.Free;
 end;
end;

function THREFLinker.LoadDataByFTP;
var TM : TMemoryStream;
begin
 result := false;
 if not Assigned(FIdFTP) then exit;
 FIdFTP.User := AHREF.UserName;
 FIdFTP.Password := AHREF.UserPwd;
 if AHREF.Port<>'' then FIdFTP.Port := StrToInt(AHREF.Port);
 TM := TMemoryStream.Create;
 try
  FIdFTP.Connect(true);
  FIdFTP.ChangeDir(AHREF.UserDir+AHREF.Path);
  FIdFTP.Get(AHREF.Document,TM);
  ALines.LoadFromStream(TM);
  AHREF.State := lsGood;
  result := true;
 finally
  TM.Free;
  FIdFTP.Disconnect;
 end;
end;

function THREFLinker.LoadDataByHTTP;
begin
 result := false;
 if not Assigned(FIdHTTP) then exit;
 FIdHTTP.Request.Username := AHREF.UserName;
 FIdHTTP.Request.Password := AHREF.UserPwd;
 if AHREF.Port<>'' then FIdHTTP.Port := StrToInt(AHREF.Port);
 try
  ALines.Text := FIdHTTP.Get(AHREF.UniqueDocURL);
  AHREF.State := lsGood;
  result := true;
 finally
  FIdHTTP.Disconnect;
 end;
end;

function THREFLinker.LoadDataByLocal;
begin
 result:= FileExists(AHREF.UniqueDocURL);
 if result then
 begin
  ALines.LoadFromFile(AHREF.UniqueDocURL);
  AHREF.State := lsGood;
 end;
end;

function  THREFLinker.LoadDataByProtocol( AHREF : THREF; ALines:TStrings ): boolean;
var
    FP : TParseProtocol;
begin
 result:=false;
 AHREF.State := lsBroken;
 FP:= AHREF.ProtocolType;
 if (FP=kpUnused)or
    (FLocalParse and not((FP=kpLocal)or(FP=kpParent)))or
    (AHREF.GetFileType in [feIMG,feMEDIA,feJava,feAX])
    then exit;
 result:=true;
 try
  Case FP of
   kpLocal,
   kpParent : result:=LoadDataByLocal(AHREF,ALines);
   kpHTTP   : result:=LoadDataByHTTP(AHREF,ALines);
   kpFTP    : result:=LoadDataByFTP(AHREF,ALines);
  end;
 except
  result:=false;
 end;
end;

end.
