// IEShapes Component  1.0                                                    //
////////////////////////////////////////////////////////////////////////////////
// Part of Internet EDiting component                                         //
// AUTHOR       : Sergey Kanenko                                              //
// WEBSITE      : http://iedcomp.nm.ru                                        //
////////////////////////////  Based on  ////////////////////////////////////////
//   This code may be used and modified by anyone so long as  this header and //
// copyright  information remains intact.                                     //
//   The code is provided "as-is" and without warranty of any kind,           //
// expressed, implied or otherwise, including and without limitation, any     //
// warranty of merchantability or fitness for a  particular purpose.         //
//   In no event shall the author be liable for any special, incidental,      //
// indirect or consequential damages whatsoever (including, without           //
// limitation, damages for loss of profits, business interruption, loss       //
// of information, or any other loss), whether or not advised of the          //
// possibility of damage, and on any theory of liability, arising out of      //
// or in connection with the use or inability to use this software.         //
////////////////////////////////////////////////////////////////////////////////
unit IEShapes;
{$B-}

interface

uses
  Windows, Controls, Classes, Forms, ExtCtrls, Graphics, Messages, Math;

type
  TForwardMessage = (fmMouseDown, fmMouseUp);
  TIEArrowType = (atArrowIn, atArrowOut, atTerminator, atNone);
  TIEConnectionSide = (csLeft, csRight, csTop, csBottom);

  TIETextShape = class;
  TIECustomControl = class(TControl);

  TIESizingMode = (smTopLeft, smTop, smTopRight, smLeft, smRight,
    smBottomLeft, smBottom, smBottomRight, smNone);

  TIECustomShape = class(TGraphicControl)
  private
    FCanProcessMouseMsg: Boolean;
    FCaption: TIETextShape;
    FSelected: Boolean;
    FWasCovered: Boolean;
    FAlignment :  TAlignment;
    FOrigin: TPoint;
    FMoving: Boolean;
    FSizingMode: TIESizingMode;
    FSizeOrigin: TPoint;
    FSRectHeight: Integer;
    FSRectWidth: Integer;
    FMinHeight: Integer;
    FMinWidth: Integer;
    FResizeble: Boolean;
    procedure SetParent(AParent: TWinControl); override;
    procedure SetSelected(Value: Boolean); virtual;
    procedure SetShapeCaption(Value: TIETextShape); virtual;
    procedure SetAlignment(const Value: TAlignment);
  protected
    function  GetMidPoint: TPoint; virtual;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure StartMove(X, Y: Integer);
    function  IsMoved(DeltaX, DeltaY: Integer): Boolean;
    procedure EndMove;
    procedure Move(DeltaX, DeltaY: Integer);
    procedure MoveAll(DeltaX, DeltaY: Integer);
    function  GetCustomShapeAtPos(X, Y: Integer): TIECustomShape;
    procedure ResizeControl(X, Y: Integer); virtual;
    procedure DrawSizingRects;
    function  GetSizeRect(SizeRectType: TIESizingMode): TRect;
    procedure CheckForSizeRects(X, Y: Integer);
    procedure Paint; override;
    procedure PaintTo(ACanvas:TCanvas; X, Y: Integer); virtual;

    property  CanProcessMouseMsg: Boolean read FCanProcessMouseMsg write FCanProcessMouseMsg;
    property  Moving: Boolean read FMoving write FMoving;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure   PaintToCanvas(ACanvas: TCanvas); virtual;
    procedure   AlignCaption;  virtual;

    class procedure LoadFromFile(const FileName: string; ParentControl: TWinControl);
    class procedure SaveToFile(const FileName: string;ParentControl: TWinControl);
    class procedure DeleteAllShapes(ParentControl: TWinControl);
    class procedure DeleteSelectedShapes(ParentControl: TWinControl);
    class procedure UnselectAllShapes(ParentControl: TWinControl);

    property  Selected: Boolean read FSelected write SetSelected;
    property  Resizeble: Boolean read FResizeble write FResizeble;
    property  SizingMode: TIESizingMode read FSizingMode write FSizingMode;
    property  SizeRectHeight: Integer read FSRectHeight write FSRectHeight;
    property  SizeRectWidth: Integer read FSRectWidth write FSRectWidth;
    property  MinHeight: Integer read FMinHeight write FMinHeight;
    property  MinWidth: Integer read FMinWidth write FMinWidth;
    property  ShapeCaption: TIETextShape read FCaption write SetShapeCaption;
    property  Alignment: TAlignment read FAlignment write SetAlignment default taCenter;
    property  MidPoint: TPoint read GetMidPoint;

    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property PopupMenu;
 end;

  TIETextShape = class(TIECustomShape)
  private
    FText: string;
    FAutosizeble: Boolean;
    FFont: TFont;

    procedure SetText(Value: string);
    procedure SetAutoSizeble(Value: Boolean);
    procedure SetFont(Value: TFont);
    procedure FontChanged(Sender: TObject);
  protected
    procedure RefreshText;
    procedure SetParent(AParent: TWinControl); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure   SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure   PaintTo(ACanvas: TCanvas; X, Y: Integer); override;
  published
    property Text: string read FText write SetText;
    property Autosizeble: Boolean read FAutosizeble write SetAutoSizeble;
    property Font: TFont read FFont write SetFont;
  end;

  TIEBitmapShape = class(TIECustomShape)
  private
    FImages: TImageList;
    FImageIndex: Integer;

    procedure SetImages(Value: TImageList);
    procedure SetImageIndex(Value: Integer);
  protected
    procedure SetSelected(Value: Boolean); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure   PaintTo(ACanvas: TCanvas; X, Y: Integer); override;

    property Images: TImageList read FImages write SetImages;
    property ImageIndex: Integer read FImageIndex write SetImageIndex;
  end;

  TIEConnection = class(TPersistent)
  private
    FTerminatorRect : TRect;
    FTermPoint : TPoint;
    FConnPoint : TPoint;
    FShape: TControl;
    FSide: TIEConnectionSide;
    FOffset: Integer;
    FAuto: boolean;
    FArrowType : TIEArrowType;
    FBrush: TColor;
    FPen: TColor;

    procedure SetTerminatorRect(const Value: TRect);
    procedure SetSide(const Value: TIEConnectionSide);
    procedure SetShape(const Value: TControl);
    procedure SetArrowType(const Value: TIEArrowType);
  public
    constructor Create;
    procedure Assign(Source: TPersistent); override;
    function  LeftMost: TPoint;
    function  RightMost: TPoint;
    function  TopMost: TPoint;
    function  BottomMost: TPoint;
    procedure SetConnectionPoints;
    procedure PaintOn( Canvas : TCanvas; FC,FT : TPoint);

    property TerminatorRect: TRect read FTerminatorRect write SetTerminatorRect;
    property TermPoint: TPoint read FTermPoint;
    property ConnPoint: TPoint read FConnPoint;
    property Shape: TControl read FShape write SetShape;
    property Side: TIEConnectionSide read FSide write SetSide;
    property Offset: Integer read FOffset write FOffset;
    property AutoOffset: boolean read FAuto write FAuto default true;
    property BrushColor: TColor read FBrush write FBrush default clSilver;
    property LineColor: TColor read FPen write FPen default clBlack;
    property ArrowType : TIEArrowType read FArrowType write SetArrowType;
  end;

  TIEConnector = class(TIECustomShape)
  private
    FLineWidth: Integer;
    FLineColour: TColor;
    FStartConn: TIEConnection;
    FEndConn: TIEConnection;
    FMidPoint: TPoint;

    procedure SetLineWidth(Value: Integer);
    function  GetConn(Index: Integer): TIEConnection;
    procedure SetConn(Index: Integer; Value: TIEConnection);
    function  GetTermRect(Index: Integer): TRect;
    procedure SetTermRect(Index: Integer; Value: TRect);
    procedure CheckSize(var AWidth, AHeight: Integer);
    function  GetMidPoint: TPoint; override;
    procedure SetLineColour(const Value: TColor);
  protected
    function  Convert(APoint: TPoint): TPoint;
    function  IsConnected(ConnectedShape: TGraphicControl): Boolean;
    procedure SetParent(AParent: TWinControl); override;
    procedure PaintTo(ACanvas: TCanvas; X, Y: Integer); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure  SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure  SetBoundingRect;
    procedure  SetConnections(TheStartConn, TheEndConn: TIEConnection);
    property   StartTermRect: TRect index 1 read GetTermRect write SetTermRect;
    property   EndTermRect: TRect index 2 read GetTermRect write SetTermRect;
    property   LineWidth: Integer read FLineWidth write SetLineWidth default 1;
    property   LineColor: TColor read FLineColour write SetLineColour default clBlack;
    property   StartConn: TIEConnection index 1 read GetConn write SetConn;
    property   EndConn: TIEConnection index 2 read GetConn write SetConn;
  end;

procedure AutoConnectSide(Connector : TIEConnector);
procedure AutoSideAll;
procedure GetAttachedConnectors(Control : TControl; List: TList);
procedure InvalidateAttachedConnectors(Control : TControl);

implementation

uses
  SysUtils, ImgList, Dialogs, Types;
var
  ConnectorsList : TList;

procedure NoLessThan(var Value: Integer; Limit: Integer);
begin
  if Value < Limit then  Value := Limit;
end;

function RectHeight(ARect: TRect): Integer;
begin
  Result := ARect.Bottom - ARect.Top;
end;

function RectWidth(ARect: TRect): Integer;
begin
  Result := ARect.Right - ARect.Left;
end;

function InRect(X, Y: Integer; ARect: TRect): Boolean;
begin
  Result := (X >= ARect.Left) and (X <= ARect.Right) and
    (Y >= ARect.Top) and (Y <= ARect.Bottom);
end;

// --------------------------- TIECustomShape -----------------------------
constructor TIECustomShape.Create(AOwner: TComponent);
begin
  inherited;
  FCanProcessMouseMsg := True;
  FCaption := nil;
  FSelected := False;
  FWasCovered := False;
  FAlignment:= taCenter;
  FOrigin := Point(0, 0);
  FSizeOrigin := Point(0, 0);
  FSizingMode := smNone;
  FSRectHeight := 5;
  FSRectWidth := 5;
  FMinHeight := FSRectHeight;
  FMinWidth := FSRectWidth;
  FMoving := False;
  FResizeble:=true;
end;

destructor TIECustomShape.Destroy;
var
  i : Integer;
begin
  if Assigned(FCaption) then
   FCaption.Free;
  if Assigned(Parent) then
  begin
    i := 0;
    while i < Parent.ControlCount do
    begin
      if (Parent.Controls[i] is TIEConnector) and
        (TIEConnector(Parent.Controls[i]).IsConnected(Self)) then
        Parent.Controls[i].Free
      else
        Inc(i);
    end;
  end;
  inherited Destroy;
end;

procedure TIECustomShape.PaintTo(ACanvas: TCanvas; X, Y: Integer);
begin
  DrawSizingRects;
end;

procedure TIECustomShape.PaintToCanvas(ACanvas: TCanvas);
begin
 PaintTo(ACanvas, Left,Top);
end;

procedure TIECustomShape.SetShapeCaption(Value: TIETextShape);
begin
  if (Value = nil) and Assigned(FCaption) then
   begin
    FCaption.Free;
    FCaption := nil;
   end
  else
    if (Value <> FCaption) then
     begin
      FCaption := Value;
      FCaption.Parent := Self.Parent;
      if (FCaption.Left = 0) and (FCaption.Top = 0) then AlignCaption;
    end;
end;

procedure TIECustomShape.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if Assigned(FCaption) then
    FCaption.Parent := AParent;
end;

procedure TIECustomShape.SetSelected(Value: Boolean);
begin
  FSelected := Value;
  if Assigned(FCaption) then
    FCaption.SetSelected(Value);
  if Value <> FSelected then
   Invalidate;
end;

procedure TIECustomShape.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  i  : Integer;
begin
  NoLessThan(ALeft, 0);
  NoLessThan(ATop, 0);
  NoLessThan(AWidth, FMinWidth);
  NoLessThan(AHeight, FMinHeight);
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
  if not Assigned(Parent) then  Exit;
  for i := 0 to Parent.ControlCount - 1 do
   begin
    if Parent.Controls[i] is TIEConnector then
     begin
      if TIEConnector(Parent.Controls[i]).IsConnected(Self) then
        TIEConnector(Parent.Controls[i]).SetBoundingRect;
     end;
   end;
end;

procedure TIECustomShape.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
  begin
    if AComponent = FCaption then
      FCaption := nil;
  end;
end;

function TIECustomShape.GetCustomShapeAtPos(X, Y: Integer): TIECustomShape;
var
  i                 : Integer;
  Pt                : TPoint;
begin
  Result := nil;
  if not Assigned(Parent) then   Exit;
  Pt := Parent.ScreenToClient(ClientToScreen(Point(X, Y)));
  for i := 0 to Parent.ControlCount - 1 do
  begin
    if (Parent.Controls[i] <> Self) and
      (Parent.Controls[i] is TIECustomShape) and
      TIECustomShape(Parent.Controls[i]).CanProcessMouseMsg and
      InRect(Pt.X, Pt.Y, Parent.Controls[i].BoundsRect) then
    begin
      Result := TIECustomShape(Parent.Controls[i]);
      Exit;
    end;
  end;
end;

procedure TIECustomShape.SetAlignment(const Value: TAlignment);
begin
  FAlignment := Value;
  AlignCaption;
end;

procedure TIECustomShape.AlignCaption;
var
  ALeft, ATop: Integer;
begin
  if FCaption=nil then  Exit;
  ATop := Top + Height + 1;
  Case FCaption.Alignment of
   taLeftJustify : ALeft:=Left;
   taCenter : ALeft:= Left + (Width div 2) - FCaption.Width div 2;
   taRightJustify : ALeft:=Left + Width - FCaption.Width;
   else ALeft:=Left;
  end;
  FCaption.SetBounds(ALeft, ATop, FCaption.Width, FCaption.Height);
end;

function TIECustomShape.GetMidPoint: TPoint;
begin
  Result := Point(Left + Width div 2, Top + Height div 2);
end;

class procedure TIECustomShape.SaveToFile(const FileName: string;
  ParentControl: TWinControl);
var
  FS                : TFileStream;
  Writer            : TWriter;
  RealName          : string;
begin
  FS := TFileStream.Create(Filename, fmCreate or fmShareDenyWrite);
  Writer := TWriter.Create(FS, 1024);
  try
    Writer.Root := ParentControl.Owner;
    RealName := ParentControl.Name;
    ParentControl.Name := '';
    Writer.WriteComponent(ParentControl);
    ParentControl.Name := RealName;
  finally
    Writer.Free;
    FS.Free;
  end;
end;

class procedure TIECustomShape.LoadFromFile(const FileName: string;
  ParentControl: TWinControl);
var
  FS                : TFileStream;
  Reader            : TReader;
  RealName          : string;
begin
  DeleteAllShapes(ParentControl);
  FS := TFileStream.Create(Filename, fmOpenRead or fmShareDenyWrite);
  Reader := TReader.Create(FS, 1024);
  try
    RealName := ParentControl.Name;
    Reader.Root := ParentControl.Owner;
    Reader.BeginReferences;
    Reader.ReadComponent(ParentControl);
    Reader.FixupReferences;
    ParentControl.Name := RealName;
  finally
    Reader.EndReferences;
    Reader.Free;
    FS.Free;
  end;
end;

class procedure TIECustomShape.DeleteAllShapes(ParentControl: TWinControl);
var
  i : Integer;
begin
  i := 0;
  if ParentControl<> nil then
  while i < ParentControl.ControlCount do
    if ParentControl.Controls[i] is TIECustomShape then
      ParentControl.Controls[i].Free
    else
      Inc(i);
end;

class procedure TIECustomShape.DeleteSelectedShapes(ParentControl: TWinControl);
var
  i  : Integer;
begin
  i := 0;
  while i < ParentControl.ControlCount do
    if (ParentControl.Controls[i] is TIECustomShape) and
      (TIECustomShape(ParentControl.Controls[i]).Selected) then
      ParentControl.Controls[i].Free
    else
      Inc(i);
end;

class procedure TIECustomShape.UnselectAllShapes(ParentControl: TWinControl);
var
  i  : Integer;
begin
  for i := 0 to ParentControl.ControlCount - 1 do
    if ParentControl.Controls[i] is TIECustomShape then
    With TIECustomShape(ParentControl.Controls[i]) do
       Selected := False;
end;


procedure TIECustomShape.Paint;
begin
  if not Assigned(Parent) then Exit;
  PaintTo(Canvas,0,0);
  inherited Paint;
end;

function TIECustomShape.GetSizeRect(SizeRectType: TIESizingMode): TRect;
 var mx,my,rp,bp : integer;
begin
 mx := ClientRect.Left + ((ClientRect.Right - ClientRect.Left) div 2) - (FSRectWidth div 2);
 my := ClientRect.Top  + ((ClientRect.Bottom - ClientRect.Top) div 2) - (FSRectHeight div 2);
 rp := ClientRect.Right  - FSRectWidth;
 bp := ClientRect.Bottom - FSRectHeight;
 case SizeRectType of
  smTop:         Result := Bounds(mx, 0, FSRectWidth, FSRectHeight);
  smLeft:        Result := Bounds(0, my,  FSRectWidth, FSRectHeight);
  smRight:       Result := Bounds(rp, my,  FSRectWidth, FSRectHeight);
  smBottom:      Result := Bounds(mx,bp,  FSRectWidth, FSRectHeight);
  smTopLeft:     Result := Bounds(0,  0, FSRectWidth, FSRectHeight);
  smTopRight:    Result := Bounds(rp, 0, FSRectWidth, FSRectHeight);
  smBottomLeft:  Result := Bounds(0, bp, FSRectWidth, FSRectHeight);
  smBottomRight: Result := Bounds(rp,bp, FSRectWidth, FSRectHeight);
  smNone:        Result := Bounds(0, 0, 0, 0);
 end;
end;

procedure TIECustomShape.DrawSizingRects;
var
  SMode     : TIESizingMode;
  R         : TRect;
begin
  if (not FResizeble)or(not FSelected) then Exit;
  with Canvas do
   begin
      Brush.Style := bsSolid;
      Brush.Color := clGray;
      Pen.Color := clBlack;
      for SMode := smTopLeft to smBottomRight do
       begin
        R:= GetSizeRect(SMode);
        FillRect(R);
        Rectangle(R);
       end;
   end;
end;

procedure TIECustomShape.CheckForSizeRects(X, Y: Integer);
var
  SMode : TIESizingMode;
begin
  FSizingMode := smNone;
  if (not FResizeble)or(not FSelected) then Exit;
  for SMode := smTopLeft to smBottomRight do
  begin
    if InRect(X, Y, GetSizeRect(SMode)) then
     begin
      FSizingMode := SMode;
      Break;
     end;
  end;

  case FSizingMode of
    smTopLeft: Cursor := crSizeNWSE;
    smTop: Cursor := crSizeNS;
    smTopRight: Cursor := crSizeNESW;
    smLeft: Cursor := crSizeWE;
    smRight: Cursor := crSizeWE;
    smBottomLeft: Cursor := crSizeNESW;
    smBottom: Cursor := crSizeNS;
    smBottomRight: Cursor := crSizeNWSE;
  else
    Cursor := crDefault;
  end;
end;

procedure TIECustomShape.ResizeControl(X, Y: Integer);
var
  L, T, W, H, DeltaX, DeltaY: Integer;
begin
  L := Left;  T := Top;  W := Width;  H := Height;
  DeltaX := X - FSizeOrigin.X;
  DeltaY := Y - FSizeOrigin.Y;

  case FSizingMode of
    smTopLeft:
      begin
        if (L + DeltaX >= 0) and (W - DeltaX > MinWidth) then
         begin inc(L,DeltaX); dec(W,DeltaX); end;
        if (T + DeltaY >= 0) and (H - DeltaY > MinHeight) then
         begin inc(T,DeltaY); dec(H,DeltaY); end;
      end;
    smTopRight:
      begin
        inc(W,DeltaX);
        if (T + DeltaY >= 0) and (H - DeltaY > MinHeight) then
         begin inc(T,DeltaY); dec(H,DeltaY); end;
        FSizeOrigin.X := X;
      end;
    smTop:
      if (T + DeltaY >= 0) and (H - DeltaY > MinHeight) then
       begin inc(T,DeltaY); dec(H,DeltaY);  end;
    smLeft:
      if (L + DeltaX >= 0) and (W - DeltaX > MinWidth) then
       begin inc(L,DeltaX); dec(W,DeltaX);  end;
    smRight: begin inc(W,DeltaX); FSizeOrigin.X := X; end;
    smBottomLeft:
      begin
        if (L + DeltaX >= 0) and (W - DeltaX > MinWidth) then
         begin inc(L,DeltaX); dec(W,DeltaX);  end;
        inc(H,DeltaY);
        FSizeOrigin.Y := Y;
      end;
    smBottom: begin inc(H,DeltaY); FSizeOrigin.X := X; FSizeOrigin.Y := Y; end;
    smBottomRight:
      begin
        inc(W,DeltaX); inc(H, DeltaY); FSizeOrigin.X := X; FSizeOrigin.Y := Y;
      end;
    smNone: ;
  end;
  SetBounds(L, T, W, H);
end;

procedure TIECustomShape.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  TempPt            : TPoint;
  CoveredShape      : TIECustomShape;
begin
  if not FSelected then
   begin
    if (not (ssShift in Shift)) then
     UnselectAllShapes(Parent);
    FSelected   := True;
   end;
  BringToFront;
  FSizeOrigin := Point(X, Y);

  if (FSizingMode = smNone) or (Button <> mbLeft) or (ssShift in Shift) then
   begin
    FSizingMode := smNone;
    if FCanProcessMouseMsg then
     begin
      BringToFront;
      MouseCapture := True;
      inherited MouseDown(Button, Shift, X, Y);
      if Button <> mbLeft then Exit;
      StartMove(X, Y);
     end
    else
     begin
      CoveredShape := GetCustomShapeAtPos(X, Y);
      TempPt := Point(X, Y);
      MouseCapture := False;
      if CoveredShape <> nil then
       begin
        SendToBack;
        TempPt := CoveredShape.ScreenToClient(ClientToScreen(TempPt));
        CoveredShape.MouseDown(Button, Shift, TempPt.X, TempPt.Y);
        CoveredShape.FWasCovered := True;
       end
      else
        if Assigned(Parent) then
         begin
          TempPt := Parent.ScreenToClient(ClientToScreen(TempPt));
          TIECustomControl(Parent).MouseDown(Button, Shift, TempPt.X, TempPt.Y);
         end;
     end;
   end;
end;

procedure TIECustomShape.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
   FSizingMode := smNone;
  inherited MouseUp(Button, Shift, X, Y);
  if FWasCovered then
   begin
    Click;
    FWasCovered := False;
   end;
  if Button <> mbLeft
   then Exit;
  EndMove;
  if not Assigned(Parent)
   then Exit;
  Parent.Invalidate;
end;

procedure TIECustomShape.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  if Moving then
   begin
    inherited MouseMove(Shift, X, Y);
    if not (ssLeft in Shift) then
     begin
      Moving := False;
      Exit;
     end;
     if Moving then
      MoveAll(X - FOrigin.X, Y - FOrigin.Y);
   end
  else
   begin
    if (FSizingMode <> smNone) and (ssLeft in Shift) then
      ResizeControl(X, Y)
    else
      CheckForSizeRects(X, Y);
   end;
end;


procedure TIECustomShape.EndMove;
begin
  Moving := False;
  FOrigin := Point(0, 0);
end;

procedure TIECustomShape.Move(DeltaX, DeltaY: Integer);
begin
  SetBounds(Left + DeltaX, Top + DeltaY, Width, Height);
  Invalidate;
end;

procedure TIECustomShape.MoveAll(DeltaX, DeltaY: Integer);
var
  i, Pass           : Integer;
  TempControl       : TControl;
begin
  if not Assigned(Parent) then   Exit;
  for Pass := 1 to 2 do
   for i := 0 to Parent.ControlCount - 1 do
    begin
      TempControl := Parent.Controls[i];
      if TempControl is TIECustomShape then
       begin
        if (Pass = 1) then
         begin
          if (not TIECustomShape(TempControl).IsMoved(DeltaX, DeltaY)) then
           Exit
         end
        else
          if TIECustomShape(TempControl).Selected then
            TIECustomShape(TempControl).Move(DeltaX, DeltaY);
       end;
    end;
end;

procedure TIECustomShape.StartMove(X, Y: Integer);
begin
  Selected := True;
  Moving := True;
  FOrigin := Point(X, Y);
end;

function TIECustomShape.IsMoved(DeltaX, DeltaY: Integer): Boolean;
begin
  Result := True;
  if not Assigned(Parent) then Exit;
  if Selected then
  begin
    Result := (Left + DeltaX >= 0) and
      (Top + DeltaY >= 0) and
      (Left + DeltaX + Width - 1 <
      Parent.ClientRect.Right - Parent.ClientRect.Left) and
      (Top + DeltaY + Height - 1 <
      Parent.ClientRect.Bottom - Parent.ClientRect.Top);
  end;
end;
// ----------------------------- TIETextShape  ------------------------------
constructor TIETextShape.Create(AOwner: TComponent);
begin
  inherited;
  FAutosizeble := True;
  FResizeble := false;
  FText := '';
  FFont := TFont.Create;
  FFont.OnChange := FontChanged;
end;

destructor TIETextShape.Destroy;
begin
  FFont.Free;
  inherited Destroy;
end;

procedure TIETextShape.RefreshText;
var
  i, Count          : Integer;
  TempStr           : string;
begin
  FMinHeight := FSRectHeight;
  FMinWidth := FSRectWidth;
  TempStr := '';
  Count := 1;
  if FAutosizeble and Assigned(Parent) then
  begin
    Canvas.Font := Font;
    for i := 1 to Length(FText) do
     begin
      if FText[i] = #10 then
      begin
        FMinWidth := Max(FMinWidth, Canvas.TextWidth(TempStr));
        TempStr := '';
        Inc(Count);
      end
      else
        TempStr := TempStr + FText[i];
    end;
    if Count = 1 then FMinWidth := Max(FMinWidth, Canvas.TextWidth(FText));
    FMinHeight := Max(FMinHeight, Canvas.TextHeight(FText) * Count);
  end;

  if FSelected then
   begin
    FMinHeight:= FMinHeight+FSRectHeight*2;
    FMinWidth := FMinWidth+FSRectWidth*2;
    SetBounds(Left-FSRectWidth, Top-FSRectHeight, FMinWidth, FMinHeight);
   end
  else
    SetBounds(Left, Top, FMinWidth, FMinHeight);
end;

procedure TIETextShape.SetText(Value: string);
begin
  if FText <> Value then
   begin
    FText := Value;
    RefreshText;
   end;
end;

procedure TIETextShape.SetAutosizeble(Value: Boolean);
begin
  if FAutosizeble <> Value then
  begin
    FAutosizeble := Value;
    RefreshText;
  end;
end;

procedure TIETextShape.SetFont(Value: TFont);
begin
  FFont.Assign(Value);
end;

procedure TIETextShape.FontChanged(Sender: TObject);
begin
  RefreshText;
end;

procedure TIETextShape.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  RefreshText;
end;

procedure TIETextShape.PaintTo(ACanvas: TCanvas; X, Y: Integer);
var
  TempRect : TRect;
begin
  if not Assigned(Parent) then  Exit;
  inherited PaintTo(ACanvas,X,Y);
  ACanvas.Font := Font;
  ACanvas.Brush.Style := bsClear;

  TempRect := ClientRect;
  OffsetRect(TempRect, X, Y);
//  TempRect := BoundsRect;
  DrawText(ACanvas.Handle, PChar(FText), Length(FText), TempRect,
           DT_CENTER or DT_VCENTER or DT_NOPREFIX or DT_WORDBREAK);
end;

procedure TIETextShape.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  NoLessThan(AWidth, FMinWidth);
  NoLessThan(AHeight, FMinHeight);
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

// ---------------------------- TIEBitmapShape ------------------------------

constructor TIEBitmapShape.Create(AOwner: TComponent);
begin
  inherited;
  FImages := nil;
  FImageIndex := 0;
  FResizeble := false;
end;

procedure TIEBitmapShape.SetSelected(Value: Boolean);
begin
  inherited SetSelected(Value);
  if Value <> FSelected then
   Invalidate;
end;

procedure TIEBitmapShape.SetImages(Value: TImageList);
begin {SetImages}
  if Value <> FImages then
   begin
    FImages := Value;
    if FImages <> nil then
      SetBounds(Left, Top, FImages.Width, FImages.Height);
   end;
end;

procedure TIEBitmapShape.SetImageIndex(Value: Integer);
begin
  if Value <> FImageIndex then
   begin
    FImageIndex := Value;
    Invalidate;
   end;
end;

procedure TIEBitmapShape.PaintTo(ACanvas: TCanvas; X, Y: Integer);
begin
  if (not Assigned(Parent)) or
    (not Assigned(FImages)) or
    (FImageIndex < 0) or
    (FImageIndex >= FImages.Count) then  Exit;

  FImages.Draw(ACanvas, X, Y, FImageIndex);
  ACanvas.Pen.Style := psDot;
  ACanvas.Brush.Style := bsClear;
  if Selected then
    ACanvas.Pen.Mode := pmNot
  else
    ACanvas.Pen.Mode := pmNop;

  ACanvas.Polyline([Point(X,Y),
                    Point(X+Width-1,Y),
                    Point(X+Width-1,Y+Height-1),
                    Point(X,Y+Height-1),
                    Point(X,Y)]);
end;

procedure TIEBitmapShape.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
  begin
    if AComponent = FImages then
      FImages := nil;
  end;
end;

// ----------------------------- TIEConnection ------------------------------
constructor TIEConnection.Create;
begin
  inherited;
  FShape := nil;
  FSide := csRight;
  FOffset := 0;
  FAuto:=true;
  FTerminatorRect:= Rect(0,0,15,9);
  FArrowType:= atArrowIn;
  FBrush:=clSilver;
  FPen:=clBlack;
end;

procedure TIEConnection.Assign(Source: TPersistent);
begin
  if Source is TIEConnection then
   begin
    FShape := TIEConnection(Source).FShape;
    FSide := TIEConnection(Source).FSide;
    FOffset := TIEConnection(Source).FOffset;
   end
  else
    inherited Assign(Source);
end;

Procedure TIEConnection.SetConnectionPoints;
var
  X, Y, W, XX, YY : Integer;
begin
  FTermPoint := Point(0, 0);
  FConnPoint := Point(0, 0);
  if FShape = nil then Exit;
  W := FTerminatorRect.Right - FTerminatorRect.Left;
  if FAuto then
   case FSide of
     csLeft, csRight: FOffset := FShape.Height div 2;
     csTop, csBottom: FOffset := FShape.Width div 2;
   end;

  case FSide of
      csLeft: begin
               X := FShape.Left - 1;     Y  := FShape.Top + FOffset;
               XX:= FShape.Left - W - 1; YY := Y;
              end;
     csRight: begin
               X := FShape.Left + FShape.Width + 1;     Y  := FShape.Top + FOffset;
               XX:= FShape.Left + FShape.Width + W + 1; YY := Y;
              end;
       csTop: begin
               X := FShape.Left + FOffset;  Y := FShape.Top - 1;
              XX := X;                     YY := FShape.Top - W - 1;
              end;
    csBottom: begin
               X := FShape.Left + FOffset; Y := FShape.Top + FShape.Height + 1;
              XX := X;                     YY:= FShape.Top + FShape.Height + W + 1;
              end;
  end;
  FTermPoint := Point(X,Y);
  FConnPoint := Point(XX,YY);
end;

procedure TIEConnection.PaintOn(Canvas : TCanvas; FC, FT : TPoint);
var
  PointPt, Corner1Pt, Corner2Pt: TPoint;
  HH,WW : integer;
begin
  inherited;

  Canvas.Brush.Color := FBrush;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := FPen;

   with Canvas do
    Case FArrowType of
     atArrowIn,
     atArrowOut: begin
                   if (FArrowType=atArrowIn) then
                    begin
                     PenPos := FC;
                     PointPt := FT;
                     Corner1Pt := FC;
                     Corner2Pt := FC;
                     LineTo(FT.X, FT.Y);
                    end
                     else
                       begin
                        PenPos := FT;
                        PointPt := FC;
                        Corner1Pt := FT;
                        Corner2Pt := FT;
                        LineTo(FC.X, FC.Y);
                       end;

                    HH := (FTerminatorRect.Bottom - FTerminatorRect.Top) div 2;
                    WW := (FTerminatorRect.Right - FTerminatorRect.Left) div 2;
                    if Corner1Pt.X < PointPt.X then
                    begin
                      Inc(Corner1Pt.X, WW); Inc(Corner2Pt.X, WW);
                      Dec(Corner1Pt.Y, HH); Inc(Corner2Pt.Y, HH);
                    end
                    else
                      if Corner1Pt.X > PointPt.X then
                      begin
                        Dec(Corner1Pt.X, WW); Dec(Corner2Pt.X, WW);
                        Dec(Corner1Pt.Y, HH); Inc(Corner2Pt.Y, HH);
                      end
                      else
                        if Corner1Pt.Y < PointPt.Y then
                        begin
                          Inc(Corner1Pt.Y, WW); Inc(Corner2Pt.Y, WW);
                          Dec(Corner1Pt.X, HH); Inc(Corner2Pt.X, HH);
                        end
                        else
                        begin
                          Dec(Corner1Pt.Y, WW); Dec(Corner2Pt.Y, WW);
                          Dec(Corner1Pt.X, HH); Inc(Corner2Pt.X, HH);
                        end;
                    Polygon([PointPt, Corner1Pt, Corner2Pt]);
                  end;
       atTerminator: begin
                       PenPos := FC; LineTo(FT.X,FT.Y);
                     end;
       atNone: ;
   end;
end;

procedure TIEConnection.SetArrowType(const Value: TIEArrowType);
begin
  FArrowType := Value;
end;

function TIEConnection.LeftMost: TPoint;
begin
  Result := FTermPoint;
  if FShape = nil then Exit;
  case FSide of
     csLeft: Result.X := FShape.Left - RectWidth(TerminatorRect);
    csRight: Result.X := FShape.Left + FShape.Width;
   csTop,
   csBottom: Result.X := FShape.Left + FOffset - (RectHeight(TerminatorRect) div 2);
  end;
end;

function TIEConnection.RightMost: TPoint;
begin
  Result := FTermPoint;
  if FShape = nil then  Exit;
  case FSide of
    csLeft: Result.X := FShape.Left - 1;
    csRight: Result.X := FShape.Left + FShape.Width - 1 + RectWidth(TerminatorRect);
    csTop,
    csBottom: Result.X := FShape.Left + FOffset + (RectHeight(TerminatorRect) div 2);
  end;
end;

function TIEConnection.TopMost: TPoint;
begin
  Result := FTermPoint;
  if FShape = nil then Exit;
  case FSide of
    csLeft,
    csRight: Result.Y := FShape.Top + FOffset - (RectHeight(TerminatorRect) div 2);
    csTop: Result.Y := FShape.Top - RectWidth(TerminatorRect) - 1;
    csBottom: Result.Y := FShape.Top + FShape.Height;
  end;
end;

function TIEConnection.BottomMost: TPoint;
begin
  Result := FTermPoint;
  if FShape = nil then  Exit;
  case FSide of
    csLeft,
    csRight: Result.Y := FShape.Top + FOffset + (RectHeight(TerminatorRect) div 2);
    csTop: Result.Y := FShape.Top - 1;
    csBottom: Result.Y := FShape.Top + FShape.Height + RectWidth(TerminatorRect);
  end;
end;

procedure TIEConnection.SetTerminatorRect(const Value: TRect);
begin
  FTerminatorRect := Value;
  SetConnectionPoints;
end;

procedure TIEConnection.SetSide(const Value: TIEConnectionSide);
begin
  FSide := Value;
  SetConnectionPoints;
end;

procedure TIEConnection.SetShape(const Value: TControl);
begin
  FShape := Value;
  SetConnectionPoints;
end;

// ----------------------------- TIEConnector -------------------------------
constructor TIEConnector.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLineWidth := 1;
  FLineColour := clBlack;
  FStartConn := TIEConnection.Create;
  FEndConn := TIEConnection.Create;
  FMidPoint := Point(0, 0);
  FCanProcessMouseMsg:=false;
  if Assigned(ConnectorsList) then
    ConnectorsList.Add(self);
end;

destructor TIEConnector.Destroy;
var idx : integer;
begin
  if Assigned(FStartConn) then FStartConn.Free;
  if Assigned(FEndConn) then FEndConn.Free;
  if Assigned(ConnectorsList) then
   begin
    idx:= ConnectorsList.IndexOf(Self);
    if idx<>-1 then ConnectorsList.Delete(idx);
   end;
  inherited;
end;

procedure TIEConnector.PaintTo(ACanvas: TCanvas; X, Y: Integer);
var
  SC,ST,EC,ET : TPoint;
begin
  SetBoundingRect;
  if not Assigned(Parent) then  Exit;
  if Assigned(FStartConn.Shape) and Assigned(FEndConn.Shape) then
   begin
    SC := Convert(FStartConn.ConnPoint); inc(SC.X,X);inc(SC.Y,Y);
    ST := Convert(FStartConn.TermPoint); inc(ST.X,X);inc(ST.Y,Y);
    EC := Convert(FEndConn.ConnPoint);   inc(EC.X,X);inc(EC.Y,Y);
    ET := Convert(FEndConn.TermPoint);   inc(ET.X,X);inc(ET.Y,Y);
   with ACanvas do
     begin
      Brush.Style := bsClear;
      Pen.Width := FLineWidth;
      Pen.Color := FLineColour;
      PenPos := SC;       //From
      LineTo(EC.X, EC.Y); //To
     end;
    FStartConn.PaintOn(ACanvas,SC,ST);
    FEndConn.PaintOn(ACanvas,EC,ET);
   end;
end;

procedure TIEConnector.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if csDestroying in ComponentState then exit;
  if Operation = opRemove then
   begin
    if AComponent = FStartConn.FShape then FStartConn.FShape := nil;
    if AComponent = FEndConn.FShape then FEndConn.FShape := nil;
    if AComponent = FCaption then FCaption := nil;
   end;
end;

procedure TIEConnector.CheckSize(var AWidth, AHeight: Integer);
begin
  NoLessThan(AHeight, FLineWidth);
  NoLessThan(AWidth, FLineWidth);
  NoLessThan(AHeight, RectHeight(FStartConn.TerminatorRect));
  NoLessThan(AWidth, RectWidth(FStartConn.TerminatorRect));
  NoLessThan(AHeight, RectHeight(FEndConn.TerminatorRect));
  NoLessThan(AWidth, RectWidth(FEndConn.TerminatorRect));
end;

procedure TIEConnector.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  CheckSize(AWidth, AHeight);
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
  AlignCaption;
end;

procedure TIEConnector.SetBoundingRect;
var
  ALeft, ATop, AWidth, AHeight: Integer;
begin
  if Assigned(FStartConn)and
     Assigned(FEndConn) and
     Assigned(FStartConn.Shape) and
     Assigned(FEndConn.Shape) then
  begin
   FStartConn.SetConnectionPoints;  FEndConn.SetConnectionPoints;
   ALeft := Min(FStartConn.LeftMost.X, FEndConn.LeftMost.X);
   ATop := Min(FStartConn.TopMost.Y, FEndConn.TopMost.Y);
   AWidth := Max(FStartConn.RightMost.X, FEndConn.RightMost.X) - ALeft + 2;
   AHeight := Max(FStartConn.BottomMost.Y, FEndConn.BottomMost.Y) - ATop + 2;
   CheckSize(AWidth, AHeight);
   UpdateBoundsRect(Rect(ALeft, ATop, ALeft + AWidth - 1, ATop + AHeight - 1));
   AlignCaption;
  end;
end;

function TIEConnector.GetMidPoint: TPoint;
var
  A, B : TPoint;
begin
  Result := Point(0, 0);
  if (not Assigned(FStartConn)) or (not Assigned(FEndConn)) then   Exit;
  A := FStartConn.ConnPoint;
  B := FEndConn.ConnPoint;
  Result := Point(Min(A.X, B.X) + Abs(A.X - B.X) div 2, Min(A.Y, B.Y) + Abs(A.Y - B.Y) div 2);
end;


procedure TIEConnector.SetLineWidth(Value: Integer);
begin
  if Value >= 1 then FLineWidth := Value;
end;

function TIEConnector.GetConn(Index: Integer): TIEConnection;
begin
  Result := nil;
  case Index of
    1: Result := FStartConn;
    2: Result := FEndConn;
  end;
end;

procedure TIEConnector.SetConn(Index: Integer; Value: TIEConnection);
begin
  case Index of
    1: FStartConn.Assign(Value);
    2: FEndConn.Assign(Value);
  end;
  SetBoundingRect;
  Invalidate;
end;

procedure TIEConnector.SetConnections(TheStartConn, TheEndConn: TIEConnection);
begin
  StartConn := TheStartConn;
  EndConn := TheEndConn;
end;

function TIEConnector.GetTermRect(Index: Integer): TRect;
begin
  case Index of
    1: Result := FStartConn.TerminatorRect;
    2: Result := FEndConn.TerminatorRect;
  end;
end;

procedure TIEConnector.SetTermRect(Index: Integer; Value: TRect);
begin
  if (Value.Right - Value.Left >= 0) and (Value.Bottom - Value.Top >= 0) then
  begin
    case Index of
      1: FStartConn.TerminatorRect := Value;
      2: FEndConn.TerminatorRect := Value;
    end;
  end;
end;

procedure TIEConnector.SetParent(AParent: TWinControl);
begin
  inherited;
  if Assigned(AParent) then
   begin
    if Assigned(FCaption) then
     FCaption.Parent := Parent;
    SetBoundingRect;
    Self.Invalidate;
   end;
end;

function TIEConnector.Convert(APoint: TPoint): TPoint;
begin
  Result := ScreenToClient(Parent.ClientToScreen(APoint));
end;

function TIEConnector.IsConnected(ConnectedShape: TGraphicControl): Boolean;
begin
  Result := (FStartConn.Shape = ConnectedShape) or (FEndConn.Shape = ConnectedShape);
end;

procedure TIEConnector.SetLineColour(const Value: TColor);
begin
  FLineColour := Value;
  if Assigned(FStartConn) then
   FStartConn.LineColor := Value;
  if Assigned(FEndConn) then
   FEndConn.LineColor := Value;
end;

// ------------------ Procedures --------------------
procedure AutoConnectSide( Connector : TIEConnector );
var SMP,EMP : TPoint;
begin
 if Assigned(Connector.StartConn.Shape) and Assigned(Connector.EndConn.Shape) then
 with Connector do
  begin
   With StartConn do
    begin
      SMP.Y := Shape.Top + Shape.Height div 2;
      SMP.X := Shape.Left + Shape.Width div 2;
    end;
   With EndConn do
    begin
      EMP.Y := Shape.Top + Shape.Height div 2;
      EMP.X := Shape.Left + Shape.Width div 2;
    end;
  if Abs(SMP.X-EMP.X)<Abs(SMP.Y-EMP.Y) then
   begin
    if SMP.Y < EMP.Y then
     begin
      StartConn.Side := csBottom;
      EndConn.Side := csTop;
     end
    else
     begin
      StartConn.Side := csTop;
      EndConn.Side := csBottom;
     end;
   end
  else
   begin
    if SMP.X < EMP.X then
     begin
      StartConn.Side := csRight;
      EndConn.Side := csLeft;
     end
    else
     begin
      StartConn.Side := csLeft;
      EndConn.Side := csRight;
     end;
   end;
  SetBoundingRect;
  Invalidate;
  end;
end;

procedure AutoSideAll;
var i : integer;
begin
 if ConnectorsList=nil then exit;
 for i:=0 to ConnectorsList.Count-1 do
  AutoConnectSide(TIEConnector(ConnectorsList[i]));
end;

procedure GetAttachedConnectors(Control : TControl; List: TList);
var i : integer;
begin
 List.Clear;
 if ConnectorsList=nil then exit;
 for i:=0 to ConnectorsList.Count-1 do
  With TIEConnector(ConnectorsList[i]) do
   if (StartConn.Shape=Control)or(EndConn.Shape=Control) then
    List.Add(ConnectorsList[i]);
end;

procedure InvalidateAttachedConnectors(Control : TControl);
var i : integer;
begin
 if ConnectorsList=nil then exit;
 for i:=0 to ConnectorsList.Count-1 do
  With TIEConnector(ConnectorsList[i]) do
   if (StartConn.Shape=Control)or(EndConn.Shape=Control) then
    SetBoundingRect;
end;

// ------------------ Initialisation and cleanup routines --------------------
procedure RegisterStorageClasses;
begin
  ConnectorsList := TList.Create;
   RegisterClasses([TIECustomShape,
                    TIECustomShape,
                    TIETextShape,
                    TIEBitmapShape,
                    TIEConnection,
                    TIEConnector]);
end;

procedure UnRegisterStorageClasses;
begin
 if Assigned(ConnectorsList) then ConnectorsList.Free;
end;



initialization
  RegisterStorageClasses;
finalization
  UnRegisterStorageClasses;
end.
